/*
 *  call.c - operations with callsigns
 *  Copyright (c)2002 - 2004 Andrej  OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "osdep.h"
#include "debug.h"
#include "main.h"

/* Stripes slash suffixes from calls */ 
char *strip_slashsfx(char *call)
{
	char *i = strrchr(call, '/');
	if (i != NULL)
		*i = '\0';
	return call;
}

uint8_t *alloc_buf(uint32_t size)
{
	DEBUG_FUNC("alloc_buf");

	/*
	 * while (buf_free+size>buf_ptr+buf_size) { buf_size+=0x10000; DWORD
	 * i=buf_free-buf_ptr; buf_ptr=(BYTE *)realloc(buf_ptr, buf_size); if
	 * (!buf_ptr) { printf("! Not enought memory\n"); exit(-2); }
	 * buf_free=buf_ptr+i; } 
	 */

	if (buf_free + size > buf_ptr + buf_size) {
		buf_ptr = (uint8_t *) malloc(buf_size);
		if (!buf_ptr) {
			printf("! Not enought memory\n");
			exit(-2);
		}
		buf_free = buf_ptr;
		mem_used += buf_size;
	}

	buf_free += size;
	return (buf_free - size);
}

struct CALL_INFO *add_call(char *call, uint32_t loc, uint8_t band)
{
	DEBUG_FUNC("add_call");

	struct CALL_INFO *c = first_call[band];

	if (strlen(call) < 3)
		return NULL;

	strupr(call);
	if (no_slashes) {
		strip_slashsfx(call);
	}
    
    int is_m = 0;
    char *cc = rindex(call, '/');
    if (cc) {
        if (strcasecmp(cc+1, "M")==0) is_m = 1;
        if (strcasecmp(cc+1, "MM")==0) is_m = 1;
    }

/*har *end = strchr(call, 0);
	if (*(end - 1) == 'P' && *(end - 2) == '/')
		*(end - 1) = 'p';*/

	while (c) {
		if (c->loc == loc && !strcmp(call, c->call))
			return c;
		c = c->next;
	}

	c = (struct CALL_INFO *) alloc_buf(sizeof(struct CALL_INFO));
	memset(c, 0, sizeof(struct CALL_INFO));

	calls++;
	strcpy(c->call, call);
	c->loc = loc;
    c->is_m = is_m;

	if (!last_call[band])
		first_call[band] = c;
	else
		last_call[band]->next = c;

	last_call[band] = c;

	return c;
}

struct CALL_INFO *find_call(char *call, uint32_t loc, uint8_t band)
{
	DEBUG_FUNC("find_call");

	struct CALL_INFO *c = first_call[band];

	strupr(call);

	while (c) {
		if (c->loc == loc && !strcmp(call, c->call))
			return c;
		c = c->next;
	}

	return NULL;
}

struct CALL_INFO *find_only_call(struct CALL_INFO *call, uint8_t band)
{
	DEBUG_FUNC("find_only_call");
	
	struct CALL_INFO *c = first_call[band];
//	strupr(call);
	
	while (c) {
		if (c != call && !strcmp(call->call, c->call))
			return c;
		c = c->next;
	}
	
	return NULL;
}

int found_in_bad_calls(const char *call)
{
	DEBUG_FUNC("found_in_bad_calls");
	
	int i;
	
	for (i = 0; i < bad_calls_cnt; i++) {
		if (!strcmp(call, bad_calls[i].bad)) {
			return i;
		}
	}
			
	return -1;
}
