
/*
    claimed - Claimed score
    Copyright (C) 2009 Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 2 as published by the Free Software Foundation.

*/

#include "osdep.h"
#include "debug.h"
#include "main.h"
#include "compare.h"
#include "claimed.h"
#include "stats.h"
#include "load.h"

void make_claimed(uint8_t band, char *band_str, uint8_t categ, 
        FILE * htm, FILE *txt,
        float maxlp, float maxqrp, int islp){

	char s[10], asl[10], pwr[50];
	struct CALL_INFO *c, **c2;
	int num = 0, i = 0;
	char loc[7], *callspace;
	char odx_call[SIZE_CALL + 1];
	int odx_QRB;
    char lpstr[20];

	DEBUG_FUNC("make_claimed");
	
    strcpy(lpstr, "");
    if (islp == 1) sprintf(lpstr, "_%d", (int)maxlp);
    if (islp == 2) sprintf(lpstr, "_%d", (int)maxqrp);

    // search for number of stations in category
    for (c = first_call[band]; c; c = c->next){
		if (c->categ != categ) continue;
        int incl = inclcall(c->pwr, maxlp, maxqrp, islp);
//      printf("claim: %-6s %3.0f %3.0f %d %d\n", c->call, c->pwr, maxpwr, islp, incl);
        if (!incl) continue;

        num++;
	}

	if (!num)
		return;

	printf("+ Generating claimed of %s, %s: ", band_str,
		   categs[categ].name);

//    printf("  maxpwr=%f islp=%d strictlp=%d num=%d ", maxpwr, islp, strictlp, num);

    fprintf(htm, "<a name=\"%d_%d%s\"></a>\n", band, categ, lpstr);
	
    fprintf(htm,
			"<h2>Category: %s, %s</h2>\n",
			band_str, categs[categ].name);
	fprintf(htm, "<table class=\"sortable\">\n");
	fprintf(htm, "<tr>\n");
	fprintf(htm, "<th>Pl.</th>\n");
	fprintf(htm, "<th>Call</th>\n");
	fprintf(htm, "<th>Score</th>\n");
	fprintf(htm, "<th>QSO</th>\n");
	fprintf(htm, "<th>Locator</th>\n");
	fprintf(htm, "<th>ASL</th>\n");
	fprintf(htm, "<th class=\"sorttable_numeric\">Power</th>\n");
	fprintf(htm, "<th>Antenna</th>\n");
	fprintf(htm, "</tr>\n");
    
    fprintf(txt, "\n\nCategory: %s, %s\n\n", band_str, categs[categ].name);
    fprintf(txt, " %3s  %-12s  %6s  %4s  %6s  %4s  %7s %s\n", "Pl.", "Call", "Score", "QSO", "Loc.", "ASL", "Power", "Antenna");

    ///
	c2 = (struct CALL_INFO **) malloc(num * sizeof(c));
        
	
    for (c = first_call[band]; c; c = c->next){
        if (!inclcall(c->pwr, maxlp, maxqrp, islp)) continue;

		if (c->categ == categ) {
			c2[i] = c;
			i++;
		}
	}
	
	qsort(c2, num, sizeof(c), compare_claimed_fcn);
	
    for (i = 0; i < num; i++) {
		num2loc(c2[i]->loc, loc);
        sprintf(s, "%i.", i + 1);
		
        if (strlen(s) < 4)
			strcat(s, space + strlen(space) + strlen(s) - 4);
		if (strlen(c2[i]->call) < 10)
			callspace = space + strlen(space) + strlen(c2[i]->call) - 10;
		else
			callspace = strchr(space, 0);

		if (c2[i]->asl)
			sprintf(asl, "%4i", c2[i]->asl);
		else
			strcpy(asl, "   ?");

        if (fpower){
            if (inclcall(c2[i]->pwr, maxlp, maxqrp, 2)) 
                strcpy(pwr, "    A");
            else if (inclcall(c2[i]->pwr, maxlp, maxqrp, 1)) 
                strcpy(pwr, "    B");
            else
                strcpy(pwr, "    C");
        }else{
            if (c2[i]->pwr)
                float_print(c2[i]->pwr, pwr);
            else
                strcpy(pwr, "   ?W");
        }

		if (!*c2[i]->ant)
			strcpy(c2[i]->ant, "?");

		*odx_call = '\0';
		odx_QRB = 0;

		if (!c2[i]->odx) {
			strcpy(odx_call, "NONE");
			odx_QRB = 0;
		} else {
			strcpy(odx_call, c2[i]->odx->r_call->call);
			odx_QRB = c2[i]->odx->qrb;
		}
		
        fprintf(htm, "<tr>\n");
		fprintf(htm,
				"<td align=right>%i.</td>\n",
				i + 1);
		fprintf(htm,
				"<td><b>%s</b></td>\n", c2[i]->call);
		fprintf(htm,
				"<td align=right>%i</td>\n",
				c2[i]->score_claimed);
		fprintf(htm,
				"<td align=right>%i</td>\n",
				c2[i]->qsos_all);
		fprintf(htm,
				"<td>%s</td>\n", loc);
		fprintf(htm,
				"<td align=right>%s</td>\n",
				asl);
		fprintf(htm,
				"<td align=right>%s</td>\n",
				pwr);
		fprintf(htm,
				"<td>%s</td>\n",
				c2[i]->ant);
		fprintf(htm, "</tr>\n");

        fprintf(txt, "%3d.  %-12s  %6d  %4d  %6s  %4s  %7s %s\n", 
                i + 1, c2[i]->call, c2[i]->score_claimed, c2[i]->qsos_all,
                loc, asl, pwr, c2[i]->ant);
    }
	fprintf(htm, "</table>\n<hr><br>");
	printf("%i stations\n", num);
}
