/*
 *  load.c - conf and edi-logs loading
 *  Copyright (c)2002-2004 Andrej  OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *  Copyright (c)2009-2011 Ladislav Vaiz OK1ZIA <ok1zia@nagano.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

//#define _ISOC99_SOURCE
#define _XOPEN_SOURCE 600

#include "osdep.h"
#include "debug.h"
#include "convert.h"
#include "main.h"
#include "load.h"
#include "call.h"
#include "cdwdb.h"
#include "diplom.h"
#include "top.h"

#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

int test_dt = -1;
int cwonly = 0;
int qsop = 1;
int wwlmult = 0;
int penboth = 1;
int checknr = 1;
int fpower = 0;
int phase = 0;
 
void make_path(char *path)
{
	DEBUG_FUNC("make_path");

	char pt[BUFF_SIZE + 1], *tmp;

#ifdef WIN_GNU
#	define MKDIR(dir)	mkdir(dir)
#else
#	define MKDIR(dir)	mkdir(dir, 0775)
#endif							/* WIN_GNU */

	strcpy(pt, path);
	DEBUG_STR("path", path);
	tmp = strstr(pt, DIR_DELIMITER);

	if (!tmp && *path)			/* path doesn't contain DIR_DELIMITER */
		MKDIR(pt);
	else
		while (tmp) {
			*tmp = 0;
			MKDIR(pt);
			strcpy(pt, path);
			tmp = strstr(tmp + 1, DIR_DELIMITER);
		}
}


float parse_power(const char *pwr, const char *band_str, const char *call){
    float ret;
    char s[100];
    const char *c;
    char *endptr;
    int i = 0;

    for(c = pwr; *c != '\0'; c++){
        if (*c == ' ') continue;
        if (*c == '\t') continue;
        if (*c == ','){
            s[i++] = '.';
            continue;
        }
        s[i++] = *c;
    }

    ret = strtof(s, &endptr);
//    printf("  '%s' %f '%s'\n", s, (double)ret, endptr);
    if (endptr == s || errno == ERANGE){
		fprintf(err, "! Can't convert SPowe='%s' ('%s') to number (%s, %s)\n", pwr, s, band_str, call);
        return 0.0;
    }
    if (!strcasecmp(endptr, "w")){
        // nothing
    }else if (!strcasecmp(endptr, "uw")){
        ret *= 0.000001;
    }else if (!strcasecmp(endptr, "mw")){
        ret *= 0.001;
    }else if (!strcasecmp(endptr, "kw")){
        ret *= 1000.0;
    }
    
//    printf("ret=%f\n", (double)ret);
    return ret;
}


int load_file(char *path, char *filename, uint8_t force_check)
{
	DEBUG_FUNC("load_file");

	char fname[BUFF_SIZE + 1], my_callsign[SIZE_CALL + 1];
	char text[BUFF_SIZE + 1], cmd[BUFF_SIZE + 1], *cmd_arg, *temp, *temp2;
	char psect[BUFF_SIZE + 1];
	char ant[SIZE_ANT + 1] = "";
	float pwr = 0;
	uint32_t asl = 0;
	int band = 0;
    char band_str[20];
	char my_loc_str[7];
	FILE *in;
	uint32_t line = 0;
	uint32_t score_claimed = 0;
	int i;
	struct CALL_INFO *my_call = NULL;
	struct QSO_INFO *qso, *last_qso = NULL;
	uint32_t my_loc = 0, qsos = 0;
	uint8_t log_entries = 0, error = 0;
	uint32_t log_qsos = 0;

	strcpy(fname, path);
	strcat(fname, filename);

	if (verbose_mode > 0)
		fprintf(inf, "Loading: %s - ", fname);

	eprintf_load = 1;
//	eprintf_filename = fname;

	in = fopen(fname, "r");

	*psect = 0;

	if (!in) {
		eprintf("!Error opening file %s\n", fname);
		return 0;
	}

	*my_callsign = 0;

	while (!feof(in)) {
		if (!fgets(text, sizeof(text) - 1, in))
			break;
        chomp(text);
		line++;
		text[199] = 0;
		if (log_entries == 0 && text[5] == '=') {
			strncpy(cmd, text, 5);
			cmd[5] = 0;
			cmd_arg = text + 6;
			temp = strchr(cmd_arg, 0);
			if (temp && temp > cmd_arg && *(temp - 1) == '\n')
				*(temp - 1) = 0;

			while (*cmd_arg == ' ' || *cmd_arg == '\t')
				cmd_arg++;
			if (!stricmp(cmd, "PCALL")) {
				if (*my_callsign) {
					eprintf("%s: PCALL= is two times in the file\n", fname);
					error = 1;
					break;
				}
				temp = cmd_arg;
				while ((*temp >= 'A' && *temp <= 'Z')
					   || (*temp >= 'a' && *temp <= 'z') || (*temp >= '0' && *temp <= '9')
					   || *temp == '/' || *temp == '\\') {
					if (*temp == '\\')
						*temp = '/';
					temp++;
				}
				*temp = 0;
				cmd_arg[SIZE_CALL] = 0;
				strcpy(my_callsign, cmd_arg);
				/*
				 * my_call=add_call(my_callsign); if (my_call->first_qso)
				 * { printf("ERROR: second log with this call\n");
				 * error=TRUE; break; } 
				 */
			} else if (!stricmp(cmd, "PWWLO")) {
				if (my_loc) {
					eprintf("%s: PWWLO= is two times in the file\n", fname);
					error = 1;
					break;
				}
				my_loc = loc2num(cmd_arg);
				if (!my_loc) {
					eprintf("%s: PWWLO= incorect locator\n", fname);
					error = 1;
					break;
				} else
					num2loc(my_loc, my_loc_str);
			} else if (!stricmp(cmd, "SANTE")) {
				strncpy(ant, cmd_arg, SIZE_ANT);
				ant[SIZE_ANT - 1] = 0;
			} else if (!stricmp(cmd, "SPOWE")) {
//				pwr = atof(cmd_arg);
                pwr = parse_power(cmd_arg, band_str, my_callsign); 
			} else if (!stricmp(cmd, "CTOSC")) {
				score_claimed = atoi(cmd_arg);
			} else if (!stricmp(cmd, "SANTH")) {
				temp = strchr(cmd_arg, ';');
				if (temp) {
					temp2 = strchr(temp + 1, ';');
					if (temp2)
						asl = atoi(temp2 + 1);
					else
						asl = atoi(temp + 1);
				}
			} else if (!stricmp(cmd, "PSECT") && !force_check) {
				strupr(cmd_arg);
				cmd_arg[19] = 0;
				temp2 = strchr(cmd_arg, ' ');
				if (temp2)
					*temp2 = 0;
				temp2 = strchr(cmd_arg, '\t');
				if (temp2)
					*temp2 = 0;
				strcpy(psect, cmd_arg);
			} else if (!stricmp(cmd, "PBAND")) {
                strncpy(band_str, cmd_arg, 18);
                band_str[19] = '\0';
				band = str2band(cmd_arg);
				if (!band) {
					eprintf
						("%s: Incorrect band PBAND=. Use only 144, 432, 1.2, ... 10, 24\n", fname);
					error = 1;
				}
			}
		} else if (!strncmpi(text, "[QSORecord", 10)) {
			log_entries = 1;
			temp = strchr(text + 10, ';');
			if (temp)
				log_qsos = atoi(temp + 1);
		} else if (log_entries == 1 && strlen(text) > 10) {
			if (!my_call) {
				/*
				 * if (!strcmp(my_rallsign, "IZ4BEH")) { my_call=FALSE; }
				 */
				if (!(*my_callsign)) {
					eprintf("%s: PCALL= must be before log entries\n", fname);
					fclose(in);
					return 0;
				}
				if (!my_loc) {
					if (!error)
						eprintf
							("%s: PWWLO= must be before log entries\n", fname);
					fclose(in);
					return 0;
				}
				if (!band) {
					if (!error)
						eprintf
							("%s: PBAND= must be before log entries\n", fname);
					fclose(in);
					return 0;
				}
				/*
				 * if (!export) { if (!error) eprintf("ERROR: PSECT= must
				 * be before log entries\n"); return; } 
				 */
				my_call = add_call(my_callsign, my_loc, band);

//                printf("'%s'\t'%p' is_m=%d\n", my_callsign, my_call, my_call->is_m);
				if (!my_call) { 
                    eprintf("%s: bad PCALL '%s'\n", fname, my_callsign); 
					error = 1;
					fclose(in);
                    return 0;
                } 
				if (my_call->first_qso) {
					eprintf
						("%s: second log with this call, locator, band\n", fname);
					error = 1;
					fclose(in);
					return 0;
				}
			}

			qso = (struct QSO_INFO *) alloc_buf(sizeof(struct QSO_INFO));
			memset(qso, 0, sizeof(struct QSO_INFO));

			qso->s_call = my_call;

			/* parse qso line */
			temp = text;
			cmd_arg = parse(&temp);
			if (*cmd_arg == '[')
				log_entries = 0;
			else {
/*				if (strlen(cmd_arg) != 6){ // date parsing 
				    eprintf("%s: line %d has incorrect DATE (%s)\n", fname, line, cmd_arg);
                }else{
					qso->dt = atoi(cmd_arg) * 10000;
                }
				cmd_arg = parse(&temp); // time parsing 
				if (strlen(cmd_arg) != 4){
				    eprintf("%s: line %d has incorrect TIME (%s)\n", fname, line, cmd_arg);
                }else{
					qso->dt += atoi(cmd_arg);
                }
*/
				qso->dt = atoi(cmd_arg);
                if (qso->dt > 999999 || qso->dt < 100000){
                    eprintf("%s: line %d incorrect date %s %u (must be without century)\n", fname, line, cmd_arg, qso->dt);
                }

                qso->dt *= 10000;
				cmd_arg = parse(&temp); /* time parsing */

                if (atoi(cmd_arg) >= phase) 
                    qso->phase = 2;
                else
                    qso->phase = 1;

                if (test_dt < 0){
                    test_dt = qso->dt;
                    eprintf("INFO: Contest date is set to %d from file %s\n", test_dt / 10000, fname);
                }else{
                    int d = test_dt - qso->dt;
                    if (d < 0) d = -d;
                    if (d > 20000) eprintf("%s: line %d date out of range (%d)\n", fname, line, qso->dt / 10000);
                }


				cmd_arg = parse(&temp); /* call parsing */
				if (!stricmp(cmd_arg, "ERROR"))
					qsos++;
				else if (*cmd_arg) {
					strncpy(my_callsign, cmd_arg, SIZE_CALL);
					my_callsign[SIZE_CALL - 1] = 0;

					cmd_arg = parse(&temp);
					i = atoi(cmd_arg);
					if (i < 1 || i > 3);
					// printf("\n- WARNING: line %d have uncorrect MODE
					// (%d) - ", line, i);
					else
						qso->s_mode = i;

					cmd_arg = parse(&temp);
					safe_strncpy0(qso->s_rst, cmd_arg, RSTLEN);
                    if (strlen(cmd_arg)<=0) eprintf("%s: RSTS empty (%s)\n", fname, my_callsign);

					cmd_arg = parse(&temp);
					qso->s_nr = atoi(cmd_arg);

					cmd_arg = parse(&temp);
					safe_strncpy(qso->r_rst, cmd_arg, RSTLEN);
                    if (strlen(cmd_arg)<=0) eprintf("%s: RSTR empty (%s)\n", fname, my_callsign);

					cmd_arg = parse(&temp);
					qso->r_nr = atoi(cmd_arg);

					cmd_arg = parse(&temp);
					cmd_arg = parse(&temp);
					qso->r_call =
						add_call(my_callsign, loc2num(cmd_arg), band);
					if (qso->r_call) {
						// if (!qso->r_call->loc && *cmd_arg) 
						// eprintf("\n- WARNING: line %d have uncorrect
						// LOCATOR (%s) - ", line, cmd_arg);

						cmd_arg = parse(&temp);
						qso->claimed_points = atoi(cmd_arg);
                        if (qso->claimed_points <= 0) 
                            qso->dupe = 1;

						cmd_arg = parse(&temp);
						cmd_arg = parse(&temp);
						cmd_arg = parse(&temp);
						cmd_arg = parse(&temp);
						if (cmd_arg && *cmd_arg == 'D') /* it's dupe? */
							qso->dupe = 1;

						qso->ok_call = qso->r_call;
						strcpy(qso->ok_rst, qso->r_rst);
						qso->ok_nr = qso->r_nr;

						all_entries++;
						qsos++;
						if (!last_qso)	// appends QSO to queue
							my_call->first_qso = qso;
						else
							last_qso->next = qso;
						last_qso = qso;
					}
				}
			}
		}
	}

	if (!error) {
		if (!my_call && *my_callsign) {
			eprintf("%s: Incorrect file format\n", fname);
			fclose(in);
			return 0;
		}

		if (!my_call) {
			eprintf("%s: PCALL= missing\n", fname);
			fclose(in);
			return 0;
		}

		if (!my_loc) {
			eprintf("%s: PWWLO= missing\n", fname);
			fclose(in);
			return 0;
		}

		if (!band) {
			eprintf("%s: PBAND= missing\n", fname);
			fclose(in);
			return 0;
		}

		if (!*psect && !force_check) {
			eprintf("%s: PSECT= missing\n", fname);
			fclose(in);
			return 0;
		}

		strcpy(my_call->ant, ant);
		my_call->asl = asl;
		my_call->pwr = pwr;
		if (force_check)
			my_call->categ = 0;
		else {
			my_call->categ = str2cat(psect, band);
//            printf("\n call='%s'  psect='%s' categ=%d\n", my_call->call, psect, my_call->categ);
			if (my_call->categ < 0) {
                my_call->categ = 0;    // not sure if has reason
				eprintf("%s: PSECT= invalid category\n", fname);
				fclose(in);
				return 0;
			}
		}
		my_call->score_claimed = score_claimed;
		strcpy(my_call->filepath, path);
		strncpy(my_call->filename, filename, sizeof(my_call->filename) - 1);
		my_call->filename[sizeof(my_call->filename) - 1] = 0;
		// my_call->loc=my_loc;
		my_call->qsos_all = qsos;

		if (qsos == 0)
			eprintf("%s: none QSO loaded\n", fname);
		else if (qsos != log_qsos)
			eprintf("%s: %d QSOs loaded but %d claimed\n", fname, qsos,
				log_qsos);
		else {
			if (band >= BAND_GHZ) {
				if (band >= BAND_GHZ_ZERO)
					sprintf(text, "%2.0f GHz", band2freq[band]);
				else
					sprintf(text, "%2.1f GHz", band2freq[band]);
			} else
				sprintf(text, "%2.0f MHz", band2freq[band]);
			if (verbose_mode > 0)
				fprintf(inf, "OK: %d QSOs of %s (%s, %s, %s, %d)\n", qsos,
					my_call->call, text, (!my_call->categ) ? "CHECK" :
					categs[(int)my_call->categ].name, my_loc_str,
					my_call->score_claimed);
			// if (!eprintf_load) fprintf(err, "\n");
		}

		// printf ("<a href=\"%s\">%s</a>\n", filename, my_call->call);
	}

	fclose(in);
	eprintf_load = 0;
	return 1;
}

int load_reku(char *path, uint8_t force_check)
{
	DEBUG_FUNC("load_reku");

	int files = 0, result;

	DIR *dp;
	struct dirent *dentry;
	struct stat fs;
	__mode_t type;
	__off_t size;
	char entry_name[BUFF_SIZE + 1];

	dp = opendir(path);

	if (!dp) {
		perror(path);
		return 0;
	}

	while ((dentry = readdir(dp)) != NULL) {

#ifdef WIN_GNU
		if (*dentry->d_name) {
#else
		/*don't load any special file */
		if (*dentry->d_name
		  /*  && (dentry->d_type == DT_DIR || dentry->d_type == DT_REG)*/) {
#endif /* WIN_GNU */

			if (strlen(path) + strlen(dentry->d_name) > BUFF_SIZE) {
				eprintf("Can't load file %s%s - path too long!\n", path, dentry->d_name);
				continue;
			}
			/* TODO: strcpy and strcat may be faster */
			/*strncpy(entry_name, path, sizeof(entry_name) - 1);
			strcat(entry_name, dentry->d_name);*/
			sprintf(entry_name, "%s%s", path, dentry->d_name);
			if (stat(entry_name, &fs) == 0) {
				type = fs.st_mode;
				size = fs.st_size;

				if (S_ISREG(type)) {	/* it's a regular file */
					/*DEBUG_STR("REGentry_name", entry_name);*/
					files += (result = load_file(entry_name, "", force_check));
					if (!result)
						fprintf(inf, "ERROR\n");
				}
				if (S_ISDIR(type) && strcmp(dentry->d_name, "..") &&
					strcmp(dentry->d_name, ".")) {	/* it's a directory */
					/*DEBUG_STR("DIRentry_name", entry_name);*/
					strcat(entry_name, DIR_DELIMITER);
					files += load_reku(entry_name, force_check);
				}
			}else{
               eprintf("Can't stat %s\n", entry_name); 
               fatals++;
            }

		}

	}

	closedir(dp);
	return files;
}


void load_path(char *pth, uint8_t force_check)
{
	DEBUG_FUNC("load_path");

	char path[BUFF_SIZE + 1];
	int files;

	printf("+ Processing path %s: \t", pth);

	strncpy(path, pth, sizeof(path) - 2);
	if (strcmp((strchr(path, '\0') - 1), DIR_DELIMITER))
		strcat(path, DIR_DELIMITER);

/*	DEBUG_STR("path", path);*/
	files = load_reku(path, force_check);

	printf(" %i files loaded\n", files);
}

void load_forcedwwl(char *fname)
{
	DEBUG_FUNC("load_forcedwwl");
	
	FILE *fd;
	char wwl[20], call[SIZE_CALL + 1];
	
	if ((fd = fopen(fname, "r")) != NULL) {
		while (!feof(fd)) {
			*wwl = '\0';
			*call = '\0';
			fscanf(fd, "%s %s", call, wwl);
			if (no_slashes) {
				strip_slashsfx(call);
			}
			if (strlen(call) > 3 && strlen(wwl) > 5) {
				forced[forcedwwl_num].wwl = loc2num(wwl);
				strcpy(forced[forcedwwl_num].call, call);
				if (verbose_mode > 1)
					fprintf(inf, "%s will be forced for %s\n", wwl, forced[forcedwwl_num].call);
				forcedwwl_num++;
				if (forcedwwl_num >= 1024)
					break;
			}
		}
		fclose(fd);
	} else
		eprintf("! Error opening forced WWLs file %s\n", fname);
}

void load_badcalls(char *fname)
{
	DEBUG_FUNC("load_badcalls");
	
	FILE *fd;
	char badcall[SIZE_CALL + 1], call[SIZE_CALL + 1];
	
	if (verbose_mode > 0)
		fprintf(inf, "Loading bad calls database: %s\n", fname);
	if ((fd = fopen(fname, "r")) != NULL) {
		while (!feof(fd)) {
			*badcall = '\0';
			*call = '\0';
			fscanf(fd, "%s %s", badcall, call);
			
			if (no_slashes) {
				strip_slashsfx(call);
				strip_slashsfx(badcall);
			}
			if (strlen(call) > 0 && strlen(badcall) > 0) {
				strcpy(bad_calls[bad_calls_cnt].bad, badcall);
				strcpy(bad_calls[bad_calls_cnt].correct, call);
				if (verbose_mode > 1)
					fprintf(inf, "Bad callsign %s will be forced to be %s\n", badcall, call);
				bad_calls_cnt++;
				if (bad_calls_cnt >= 1024)
					break;
			}
		}
		fclose(fd);
	} else
		eprintf("! Error opening forced bad callsigns file %s\n", fname);
}

void load_cfg(char *filename)
{
	DEBUG_FUNC("load_cfg");

	FILE *in;
	uint32_t line = 0;
	char text[BUFF_SIZE + 1], cmd[BUFF_SIZE + 1], *cmd_arg, *temp;
	uint8_t band;
    int categ;

	in = fopen(filename, "r");

	if (!in)
		return;

	while (!feof(in)) {
		if (!fgets(text, sizeof(text) - 1, in))
			break;
        chomp(text);
		line++;
		text[199] = 0;
		temp = strchr(text, '%');
		if (temp)
			*temp = 0;
		temp = strchr(text, 0);
		while (temp && temp > text && (*temp == 0 || isspace(*temp)))
			temp--;
		*(temp + 1) = 0;
        char *rovnase = index(text, '=');
        if (!rovnase) continue;
        int pos = rovnase-text;

//if (text[rovnaase] == '=') {
			strncpy(cmd, text, pos);
			cmd[pos] = 0;
			cmd_arg = text + pos + 1;
			temp = strchr(cmd_arg, 0);
			if (temp && temp > cmd_arg && *(temp - 1) == '\n')
				*(temp - 1) = 0;

			while (*cmd_arg == ' ' || *cmd_arg == '\t')
				cmd_arg++;
            
           //printf("\ncmd='%s'\n", cmd);


			if (!stricmp(cmd, "CTEST")) {	// contest name
				strcpy(ctest_name, cmd_arg);
			} else if (!stricmp(cmd, "CATEG") && (num_categs + 1) < MAX_CATEGS) {	// categories
				temp = cmd_arg;
				cmd_arg = parse(&temp);
				if (!strcmp(cmd_arg, "*"))
					band = 0;
				else {
					band = str2band(cmd_arg);
					if (!band) {
						printf("Error in cfg file line %i. Unknown band\n",
							line);
						band = 255;
					}
				}

				categs[num_categs].band = band;

				cmd_arg = parse(&temp);
				strupr(cmd_arg);
				categs[num_categs].psect[0] = ',';
				strncpy(categs[num_categs].psect + 1, cmd_arg, MAX_CATPSECT - 1);
				categs[num_categs].psect[MAX_CATPSECT - 2] = 0;
				strcat(categs[num_categs].psect, ",");

				cmd_arg = parse(&temp);
				strncpy(categs[num_categs].name, cmd_arg, 100);
				categs[num_categs].name[99] = 0;

				num_categs++;
			} else if (!stricmp(cmd, "LPATH")) {	// logs path
				load_path(cmd_arg, 0);
			} else if (!stricmp(cmd, "CPATH")) {	// check logs path
				load_path(cmd_arg, 1);
			} else if (!stricmp(cmd, "EPATH")) {	// error files path
				strcpy(cfg_epath, cmd_arg);

				sprintf(text, "%s", cfg_epath);
				make_path(text);
				sprintf(text, "%s%s%s", cfg_epath, DIR_DELIMITER, ERROR_FILE);
				err = fopen(text, "wt");
				if (!err) {
					printf("! Cannot create %s file\n", text);
					fclose(in);
					return;
				}

				sprintf(text, "%s%s%s", cfg_epath, DIR_DELIMITER, INFO_FILE);
				inf = fopen(text, "wt");
				if (!inf) {
					printf("! Cannot create %s file\n", text);
					fclose(in);
					return;
				}

			}
#ifdef HAVE_GLIB2
			else if (!stricmp(cmd, "CTYDB")) { /* Country database file (cty.dat) */
				printf("Loading CTY database (%s) ...\n", cmd_arg);
				if (!dw)
					dw = init_dw();
				if (dw)
					load_cty_from_file(dw, cmd_arg);
				else
					eprintf("! Error allocating memory for DW db (cty_database)\n");
			} else if (!stricmp(cmd, "DW_DB")) { /* DXCC-WWL database file */
				printf("Loading DW database (%s) ...\n", cmd_arg);
				if (!dw)
					dw = init_dw();
				if (dw)
					load_dw_from_file(dw, cmd_arg);
				else
					eprintf("! Error allocating memory for DW db (dw_database)\n");
			} else if (!stricmp(cmd, "CW_DB")) { /* Call-WWL database file */
				printf("Loading CW database (%s) ...\n", cmd_arg);
				if (!cw)
					cw = init_cw();
				if (cw)
					load_cw_from_file(cw, cmd_arg);
				else
					eprintf("! Error allocating memory for CW db (cw_database)\n");
			}
#endif /* HAVE_GLIB2 */
			else if (!stricmp(cmd, "VERBM")) { /* Search Other REFerencies */
				verbose_mode = atoi(cmd_arg);
			} else if (!stricmp(cmd, "FW_DB")) { /* Forced Wwls DataBase */
				load_forcedwwl(cmd_arg);
			} else if (!stricmp(cmd, "BC_DB")) { /* Bad Calls DataBase */
				load_badcalls(cmd_arg);
			} else if (!stricmp(cmd, "NSLSH")) { /* Strip slash suffixes from calls */
				no_slashes = atoi(cmd_arg);
			} else if (!stricmp(cmd, "DIPLOMPATH")) { /* diploma output directory */
				diplompath = strdup(cmd_arg);
				make_path(diplompath);
			} else if (!stricmp(cmd, "DIPLOMTMPL")) { /* diploma template file */
				diplomtmpl = strdup(cmd_arg);
			} else if (!stricmp(cmd, "DIPLOMPLACE")) { /* diploma output directory */
				diplomplace = atoi(cmd_arg);
			} else if (!stricmp(cmd, "STRLP")) { /* strict LP category */
				strictlp = atoi(cmd_arg);
			} else if (!stricmp(cmd, "TOP")){ /* number of TOP QSOs */
				top = atoi(cmd_arg);
			} else if (!stricmp(cmd, "CWONLY")){ /* accept only CW QSOs */
                cwonly = atoi(cmd_arg);
            } else if (!stricmp(cmd, "QSOP")){ /* QSO point method */
                qsop = atoi(cmd_arg);
            } else if (!stricmp(cmd, "WWLMULT")) { /* large WWL are multipliers */
                wwlmult = atoi(cmd_arg);
            } else if (!stricmp(cmd, "PENBOTH")) { /* penalize both sides */
                penboth = atoi(cmd_arg);
            } else if (!stricmp(cmd, "CHECKNR")) { /* check QSO number (1 for standard contests) */
                checknr = atoi(cmd_arg);
            } else if (!stricmp(cmd, "FPOWER")) { /* french politically correct power */
                fpower = atoi(cmd_arg);
            } else if (!stricmp(cmd, "PHASE")) { /* start of second phase of contest */
                phase = atoi(cmd_arg);
            }
	        for (band = 1; band < BAND_NUM; band++) {
                char key[256];
                strcpy(key, "RL_");
                band2str(key+strlen(key), band);
              //  printf("key='%s'\n", key);
                if (!stricmp(cmd, key)){
                    band2repairlimit[band] = atoi(cmd_arg);
                    //printf("set repairlimit[%s] to %d\n", key, band2repairlimit[band]);
                }
            } 
	        for (band = 1; band < BAND_NUM; band++) {
                char key[256];
                strcpy(key, "LP_");
                band2str(key+strlen(key), band);
              //  printf("key='%s'\n", key);
                if (!stricmp(cmd, key)){
                    band2lp[band] = atoi(cmd_arg);
                    //printf("set band2lp[%s] to %d\n", key, band2lp[band]);
                }

                for (categ = 1; categ < MAX_CATEGS; categ++){
                    strcpy(key, "LPBC_");
                    band2str(key+strlen(key), band);
                    sprintf(key+strlen(key), "_%d", categ);
//                    printf("key='%s'\n", key);
                    if (!stricmp(cmd, key)){
                        bandcat2lp[band][categ] = atoi(cmd_arg);
//                        printf("set bandcat2lp[%d][%d] (%s) to %d\n", band, categ, key, bandcat2lp[band][categ]);
                    }
                }
            } 
	        for (band = 1; band < BAND_NUM; band++) {
                char key[256];
                strcpy(key, "QRP_");
                band2str(key+strlen(key), band);
//                printf("key='%s'\n", key);
                if (!stricmp(cmd, key)){
                    band2qrp[band] = atoi(cmd_arg);
//                    printf("set band2qrp[%s] to %d\n", key, band2qrp[band]);
                }

                for (categ = 1; categ < MAX_CATEGS; categ++){
                    strcpy(key, "QRPBC_");
                    band2str(key+strlen(key), band);
                    sprintf(key+strlen(key), "_%d", categ);
//                    printf("key='%s'\n", key);
                    if (!stricmp(cmd, key)){
                        bandcat2qrp[band][categ] = atoi(cmd_arg);
//                        printf("set bandcat2qrp[%d][%d] (%s) to %d\n", band, categ, key, bandcat2qrp[band][categ]);
                    }
                }
            } 
		//}
	}

	fclose(in);
	
/*	if (cw && dw) {
		printf("Checking for suspicious Call-WWL in CW database ...\n");
		check_susp();
	}*/
}

char *chomp(char *str){
    int len;

    len = strlen(str);
    if (len <= 0) return str;
    if (str[len - 1] == '\n') {
        str[len - 1] = '\0';
        len--;
    }
    if (len <= 0) return str;
    if (str[len - 1] == '\r') str[len - 1] = '\0';
    return str;
}
