/*
 *  stats.c - output statistic files (refs, errs, html)
 *  Copyright (c)2002 - 2004 Andrej  OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *  Copyright (c)2009-2010 Ladislav Vaiz OK1ZIA <ok1zia@nagano.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "osdep.h"
#include "debug.h"
#include "convert.h"
#include "compare.h"
#include "main.h"
#include "qrb.h"
#include "load.h"
#include "check.h"
#include "call.h"
#include "diplom.h"
#include "stats.h"


int g_do_check;
gboolean free_gpointer_item(gpointer key, gpointer value, gpointer user_data);

void count_forcedwwl_refs(uint8_t band)
{
	struct CALL_INFO *c;
	int i;
	
	for (i = 0; i < forcedwwl_num; i++)
		if ((c = add_call(forced[i].call, forced[i].wwl, band)) && c->refs < 20000)
			c->refs += 20000;
}

void make_refs(uint8_t band)
{
	DEBUG_FUNC("make_refs");

	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q;

	printf(".");

	while (c) {
		c->refs = 0;
		q = c->first_qso;		// delete all !!!
		while (q) {
			// q->ok_call->refs=0;
			q->r_call->refs = 0;	// !!!! CHANGED 8.11.02
			q = q->next;
		}
		c = c->next;
	}

	c = first_call[band];

	while (c) {
		q = c->first_qso;
		if (q)
			c->refs += 10000;
		while (q) {
			// q->ok_call->refs++;
			q->r_call->refs++;	// !!!! CHANGED 8.11.02
			q = q->next;
		}
		c = c->next;
	}
	count_forcedwwl_refs(band);
}

void make_band_refs(uint8_t band)
{
	DEBUG_FUNC("make_band_refs");

	struct CALL_INFO *c = first_call[band], **c2;
	struct QSO_INFO *q, **q2;
	FILE *out;
	char filename[BUFF_SIZE + 1], text[BUFF_SIZE + 1], loc[7];
	float bnd = band2freq[band];
	int num = 0, i;
	char cc;

	printf(".");

	while (c) {					// clear refs
		c->refs = 0;
		q = c->first_qso;
		while (q) {
			q->r_call->refs = 0;
			q = q->next;
		}
		c = c->next;
	}

	c = first_call[band];		// make refs and num

	while (c) {
		q = c->first_qso;
		while (q) {
			if (!q->r_call->refs)
				num++;
			q->r_call->refs++;
			q = q->next;
		}
		c = c->next;
	}

	c = first_call[band];		// clear refs

	while (c) {
		q = c->first_qso;
		while (q) {
			q->r_call->refs = 0;
			q = q->next;
		}
		c = c->next;
	}


	q2 = (struct QSO_INFO **) malloc(num * sizeof(q));	// make refs again 
	// and
	// fill sort buffer

	i = 0;
	c = first_call[band];
	while (c) {
		q = c->first_qso;
		while (q) {
			if (!q->r_call->refs)
				q2[i++] = q;
			q->r_call->refs++;
			q = q->next;
		}
		c = c->next;
	}

	qsort(q2, num, sizeof(q), compare_fcn2);

	if (band >= BAND_GHZ && band < BAND_GHZ_ZERO)
		bnd *= 1000;

	if (band >= BAND_GHZ_ZERO)
		sprintf(filename, "%s%slx_G%i.ref", cfg_epath, DIR_DELIMITER,
				(int) bnd);
	else
		sprintf(filename, "%s%slx_%04i.ref", cfg_epath, DIR_DELIMITER,
				(int) bnd);
	out = fopen(filename, "wt");
	if (!out)
		return;

	if (band >= BAND_GHZ) {
		if (band >= BAND_GHZ_ZERO)
			sprintf(text, "%2.0f GHz", band2freq[band]);
		else
			sprintf(text, "%2.1f GHz", band2freq[band]);
	} else
		sprintf(text, "%2.0f MHz", band2freq[band]);

	text[8] = 0;

	fprintf(out, "Referencies on %s band\n\n", text);
	fprintf(out, "Call            Loc.   Log Refs  Status\n");


	for (i = 0; i < num; i++) {
		num2loc(q2[i]->r_call->loc, loc);
		if (q2[i]->r_call->first_qso)
			cc = 'X';
		else
			cc = ' ';
		fprintf(out, "%-16s%6s %c%6i - ", q2[i]->r_call->call, loc, cc,
				q2[i]->r_call->refs);
		if (q2[i]->r_call != q2[i]->ok_call) {
			num2loc(q2[i]->ok_call->loc, loc);
			if (q2[i]->ok_call->first_qso)
				cc = 'X';
			else
				cc = ' ';
			fprintf(out, "REPAIR: %-16s%6s %c%6i\n", q2[i]->ok_call->call,
					loc, cc, q2[i]->ok_call->refs);
		} else {
			fprintf(out, "OK\n");
		}
	}

	free(q2);

	c = first_call[band];		// clear refs

	while (c) {
		c->refs = 0;
		c->temp = 0;
		c = c->next;
	}

	num = 0;
	c = first_call[band];		// refs=num of QSO with log
	// temp=num of err QSO with log
	while (c) {
		q = c->first_qso;
		if (q /* && c->categ*/)
			num++;
		while (q) {
			if (q->ok_call->first_qso) {
				q->ok_call->refs++;
				if (!good_qso(q))
					q->ok_call->temp++;
			}
			q = q->next;
		}
		c = c->next;
	}

	c2 = (struct CALL_INFO **) malloc(num * sizeof(c));
	i = 0;

	c = first_call[band];

	while (c) {
		if (c->first_qso /* && c->categ*/)
			c2[i++] = c;
		c = c->next;
	}

	qsort(c2, num, sizeof(c), compare_fcn3);

	fprintf(out, "\nValid contacts of counterpart stations:\n\n");
	fprintf(out, "Call             QSOs  QSOs  Logged  Errors\n");
	fprintf(out, "                 all   log.  QSOs %%  %%\n");

	for (i = 0; i < num; i++) {
		fprintf(out, "%-16s %4i %4i   %5.1f   %5.1f\n", c2[i]->call,
				c2[i]->qsos_all, c2[i]->refs,
				c2[i]->qsos_all ? ((float) c2[i]->refs * 100 /
									c2[i]->qsos_all) : 0,
				c2[i]->refs ? ((float) c2[i]->temp * 100 /
									c2[i]->refs) : 0);
	}

	free(c2);

	fclose(out);
}

void err_href1(FILE *htm, struct CALL_INFO *call){
    char filename[BUFF_SIZE + 1], *tmp, *c;
    int slashes = 0, i;

    if (strlen(call->filepath) == 0) return;
    if (!call->first_qso) return; // no log
    if (!call->categ) return; // check log
    
//    sprintf(filename, "%s%s%s", cfg_epath, DIR_DELIMITER, call->filepath);
    strcpy(filename, call->filepath);
    strcat(filename, call->filename);
    for (c = filename; *c != '\0'; c++) if (*c == '/' || *c == '\\') slashes++; 
    tmp = strrchr(filename, '.');
    if (tmp) *tmp = 0;
    fprintf(htm, "<a href=\"");
    for (i = 0; i < slashes; i++) fprintf(htm, "../");
    fprintf(htm, "%s.htm\">", filename);
}

void err_href2(FILE *htm, struct CALL_INFO *call){

    if (strlen(call->filepath) == 0) return;
    if (!call->first_qso) return; // no log
    if (!call->categ) return; // check log
    fprintf(htm, "</a>");
}

void make_err_files(uint8_t band, char *band_str)
{
	DEBUG_FUNC("make_err_files");

	struct CALL_INFO *c = first_call[band];
	struct QSO_INFO *q, *odx;
	char loc[7];
	uint32_t dis, score, sumqrb;
	/*uint32_t*/ int lost; /* was int */
	FILE *htm;
	char filename[BUFF_SIZE + 1], *tmp;
	uint32_t qsos;
	uint8_t err_rnz, err_snz, err_nil, err_dupe10, err_wnc, err_irsts, err_ddupe;
    GHashTable *multhash;

	// printf("+ Generating ERR files: ");

	printf(".");

    multhash = g_hash_table_new(g_str_hash, g_str_equal);

	for (c = first_call[band]; c != NULL; c = c->next) {
		if (!c->first_qso) continue;
        if (!c->categ) continue;

        score = 0;
        sumqrb = 0;
        lost = 0;
        qsos = 0;
        odx = NULL;
        g_hash_table_foreach_remove(multhash, free_gpointer_item, NULL);
        if (wwlmult == 2){
            num2loc(c->loc, loc);
            loc[4]='\0';
            g_hash_table_insert(multhash, g_strdup(loc), NULL);
        }

        sprintf(filename, "%s%s%s", cfg_epath, DIR_DELIMITER,
                c->filepath);
        make_path(filename);
        strcat(filename, c->filename);
        tmp = strrchr(filename, '.');
        if (tmp)
            *tmp = 0;
        sprintf(c->filename_err, "%s.htm", filename);
        htm = fopen(c->filename_err, "wt");

        if (!htm) {
            fprintf(err, "! Error create error file: %s\n", c->filename_err);
            continue;
        } 
     
        fprintf(htm,
                "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
        fprintf(htm,
                "<html>\n"
                "<head>\n"
                "<style type=\"text/css\">\n"
                "body {\n"
                "    background: #f8f8f8;\n"
                "    color: #000000;\n"
                "    font-size: 12pt;\n"
                "    font-family: Tahoma, Helvetica, Arial, Arial CE, sans-serif;\n"
                "}\n"
                "table.errortable {\n"
                "    border: black 2px solid;\n"
                "    background: #f0f0f0;\n"
                "}\n"
                "table.errortable thead {\n"
                "    color: blue;\n"
                "    background: #d0d0d0;\n"
                "    cursor: pointer;\n"
                "}\n"
                "table.errortable td{\n"
                "   padding-left: 4px;\n"    
                "   padding-right: 4px;\n"    
                "   vertical-align: top;\n"
                "}\n"
                "span.bad{\n"
                "   color: red;\n"
                "}\n"
                "span.good{\n"
                "   color: green;\n"
                "}\n"
                "span.normal{\n"
                "   color: black;\n"
                "}\n"
                "</style>\n"
                "<title>Error log: %s</title>\n"
                "<script type=\"text/javascript\" src=\"../sorttable.js\"></script>\n"
                "<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-2\">"
                "</head>\n"
                "<body>\n",
                ctest_name);
        // fprintf(htm, "<html>\n<body>\n");
        fprintf(htm, "<h1>Error log: %s</h1>\n", ctest_name);

        num2loc(c->loc, loc);
        fprintf(htm, "Xtation: %s<br>\n", c->call);
        fprintf(htm, "Locator: %s<br>\n", loc);
        fprintf(htm, "Band: %s<br>\n", band_str);
        fprintf(htm, "Category:  %s<br>\n\n", categs[(int)c->categ].name);
        fprintf(htm, "<table class=\"errortable\">\n");
        fprintf(htm, "<tr><th>Call-S</th><th>RST-S</th><th>NR-S</th><th>LOC-S</th>");
        fprintf(htm, "    <th>Call-R</th><th>RST-R</th><th>NR-R</th><th>LOC-R</th>");
        fprintf(htm, "<th>Oth.err</th><th>Chk</th><th>Found by</th><th>Lost</th>%s</tr>\n", phase ? "<th>PH</th>" : "");

        err_rnz = 0;
        err_snz = 0;
        err_nil = 0;
        err_dupe10 = 0;
        err_wnc = 0;
        err_irsts = 0;
        err_ddupe = 0;
        if (verbose_mode >= 4) printf("make_err_files\n");

        for (q = c->first_qso; q != NULL; q = q->next) {
            int tr = 0;
//            if (verbose_mode >= 4 && strcmp(q->s_call->call, "OK7O")==0 && strcmp(q->ok_call->call, "EI3KD")==0) tr = 1;
 //           printf("%s %s ", q->s_call->call, q->ok_call->call);

            recalc_stats(q, c->loc);
            dis = q->points;
            if (q->dupe){
                if (tr) printf("is dupe\n");
                fprintf(htm, "<tr>\n");
                fprintf(htm, "<td>%s</td>\n", q->s_call->call);
                fprintf(htm, "<td>%s</td>\n", q->s_rst);
                fprintf(htm, "<td>%03d</td>\n", q->s_nr);
                num2loc(q->s_call->loc, loc);
                fprintf(htm, "<td>%s</td>\n", loc);
                fprintf(htm, "<td>%s</td>", q->r_call->call);
                fprintf(htm, "<td>%s</td>\n", q->r_rst);
                fprintf(htm, "<td>%03d</td>\n", q->r_nr);
                num2loc(q->r_call->loc, loc);
                fprintf(htm, "<td>%s</td>\n", loc);
                fprintf(htm, "<td><a href='#DDUPE'>DDUPE</a></td>");
                err_ddupe = 1;
                if (q->ok_call->first_qso)
                    fprintf(htm, "<td>Y</td>\n ");
                else
                    fprintf(htm, "<td></td>\n");
                fprintf(htm, "<td></td>\n"); // found by
                fprintf(htm, "<td align='right'>-0</td>\n"); // lost
                fprintf(htm, "</tr>\n");

            } else if (good_qso(q)) {
                if (tr) printf("good qso\n");
                score += q->points;
                sumqrb += q->qrb;
                qsos++;
                if (!odx || odx->qrb < dis) odx = q;

                if (wwlmult){
                    num2loc(q->r_call->loc, loc);
                    loc[4] = '\0';
                    if (!g_hash_table_lookup(multhash, loc)){
                        g_hash_table_insert(multhash, g_strdup(loc), NULL);
                    }
                }
            } else {
                if (tr) printf("bad qso\n");
                fprintf(htm, "<tr>\n");
                if (q->ok_qso && strcmp(q->s_call->call, q->ok_qso->r_call->call) != 0){
                    fprintf(htm, "<td><span class='good'>%s</span><br><span class='bad'>%s</span></td>\n", q->s_call->call, q->ok_qso->r_call->call);
                }else{
                    fprintf(htm, "<td>%s</td>\n", q->s_call->call);
                }
                if (q->ok_qso && strcasecmp(q->s_rst, q->ok_qso->r_rst) != 0){
                    if (valid_rst(q->s_rst))
                        fprintf(htm, "<td><span class='good'>%s</span>\n", q->s_rst);
                    else
                        fprintf(htm, "<td><span class='bad'>%s</span>\n", q->s_rst);
                    fprintf(htm, "<br><span class='bad'>%s</span></td>\n", q->ok_qso->r_rst);
                }else{ // same rst
                    if (valid_rst(q->s_rst))
                        fprintf(htm, "<td>%s</td>\n", q->s_rst);
                    else
                        fprintf(htm, "<td><span class='bad'>%s</span></td>\n", q->s_rst);
                } 
                if (checknr && q->ok_qso && q->s_nr != q->ok_qso->r_nr){
                    fprintf(htm, "<td><span class='good'>%03d</span><br><span class='bad'>%03d</span></td>\n", q->s_nr, q->ok_qso->r_nr);
                }else{
                    fprintf(htm, "<td>%03d</td>\n", q->s_nr);
                }                         
                if (q->ok_qso && q->s_call->loc != q->ok_qso->r_call->loc){
                    num2loc(q->s_call->loc, loc);
                    fprintf(htm, "<td><span class='good'>%s</span>", loc);
                    num2loc(q->ok_qso->r_call->loc, loc);
                    fprintf(htm, "<br><span class='bad'>%s</span></td>\n", loc);
                }else{
                    num2loc(q->s_call->loc, loc);
                    fprintf(htm, "<td>%s</td>\n", loc);
                }                              
                
                if (strcmp(q->r_call->call, q->ok_call->call) != 0){
                    fprintf(htm, "<td>");
                    err_href1(htm, q->r_call);
                    fprintf(htm, "<span class='bad'>%s", q->r_call->call);
                    if (q->r_call->refs >= 20000)
                        fprintf(htm, "&nbsp;(forced)");
                    else if (q->r_call->refs >= 10000)
                        fprintf(htm, "&nbsp;(log)");
                    else
                        fprintf(htm, "&nbsp;(%dx)", q->r_call->refs);
                    fprintf(htm, "</span>");
                    err_href2(htm, q->r_call);
                    fprintf(htm, "<br>");
                    err_href1(htm, q->ok_call);
                    fprintf(htm, "<span class='good'>%s", q->ok_call->call);
                    if (q->ok_call->refs >= 20000)
                        fprintf(htm, "&nbsp;(forced)");
                    else if (q->ok_call->refs >= 10000)
                        fprintf(htm, "&nbsp;(log)");
                    else
                        fprintf(htm, "&nbsp;(%dx)", q->ok_call->refs);
                    fprintf(htm, "</span>");
                    err_href2(htm, q->ok_call);
                    fprintf(htm, "</td>\n");

                    if (levenshtein(q->r_call->call, q->ok_call->call) > 2){
                        char loc1[7];
                        num2loc(q->r_call->loc, loc1);
                        fprintf(err, "Warning %6s %s: calls %s->%s are too different. Maybe %s %s was active in contest?\n", 
                            c->call, band_str, q->r_call->call, q->ok_call->call, q->r_call->call, loc1);
                    }
                }else{
                    fprintf(htm, "<td>");
                    err_href1(htm, q->r_call);
                    fprintf(htm, "<span class='normal'>%s</span>", q->r_call->call);
                    err_href2(htm, q->r_call);
                    fprintf(htm, "</td>\n");
                }
                if (strcasecmp(q->r_rst, q->ok_rst) != 0){
                    fprintf(htm, "<td><span class='bad'>%s</span><br>\n", q->r_rst);
                    if (valid_rst(q->ok_rst))
                        fprintf(htm, "<span class='good'>%s</span></td>\n", q->ok_rst);
                    else
                        fprintf(htm, "<span class='bad'>%s</span></td>\n", q->ok_rst);
                }else{
                    if (valid_rst(q->r_rst))
                        fprintf(htm, "<td>%s</td>\n", q->r_rst);
                    else
                        fprintf(htm, "<td><span class='bad'>%s</span></td>\n", q->r_rst);
                } 
                if (checknr && q->r_nr !=  q->ok_nr){
                    fprintf(htm, "<td><span class='bad'>%03d</span><br><span class='good'>%03d</span></td>\n", q->r_nr, q->ok_nr);
                }else{
                    fprintf(htm, "<td>%03d</td>\n", q->r_nr);
                }                         
                if (q->r_call->loc != q->ok_call->loc){
                    num2loc(q->r_call->loc, loc);
                    fprintf(htm, "<td><span class='bad'>%s", loc);
                    if (q->r_call->refs >= 20000)
                        fprintf(htm, "&nbsp;(forced)");
                    else if (q->r_call->refs >= 10000)
                        fprintf(htm, "&nbsp;(log)");
                    else
                        fprintf(htm, "&nbsp;(%dx)", q->r_call->refs);
                    fprintf(htm, "</span><br>");
                    num2loc(q->ok_call->loc, loc);
                    fprintf(htm, "<span class='good'>%s", loc);
                    if (q->ok_call->refs >= 20000)
                        fprintf(htm, "&nbsp;(forced)");
                    else if (q->ok_call->refs >= 10000)
                        fprintf(htm, "&nbsp;(log)");
                    else
                        fprintf(htm, "&nbsp;(%dx)", q->ok_call->refs);
                    fprintf(htm, "</span></td>\n");
                }else{
                    num2loc(q->r_call->loc, loc);
                    fprintf(htm, "<td>%s</td>\n", loc);
                }                              

                // ///////other error
                fprintf(htm, "<td>");
                if (q->pass == 100) {
                    fprintf(htm, "<a href='#NIL'>NIL</a>");
                    if (q->nil_qso && q->r_call == q->ok_call) // pokud je to chyba ve znacce, tak znacka u NIL jen mate
                        fprintf(htm, "<br><span class='good'>%s</span>", q->nil_qso->ok_call->call);
                    err_nil = 1;
                } else if (checknr && !q->ok_nr) {
                    fprintf(htm, "<a href='#RNZ'>RNZ</a>");
                    err_rnz = 1;
                } else if (checknr && !q->s_nr) {
                    fprintf(htm, "<a href='#SNZ'>SNZ</a>");
                    err_snz = 1;
                } else if (!strcmp(q->r_call->call, q->ok_call->call)
                       && !q->dupe && q->claimed_points && q->pass == 101) {
                    score -= dis * 9;	// if has correct call -> disqualification 10x
                    dis *= 10;
                    fprintf(htm, "<a href='#DUPEx10'>DUPEx10</a>");
                    err_dupe10 = 1;
                } else if (q->pass == 101 || q->dupe) {
                    if (!q->claimed_points)
                        dis = 0; // no points lost (claimed 0)
                    fprintf(htm, "DUPE");
                    //sprintf(text+strlen(text), " DUPE q->pass=%d, q->dupe=%d ", q->pass, q->dupe);
                    if (strcmp(q->ok_call->call, q->r_call->call)){
                        char loc1[7], loc2[7];
                        num2loc(q->r_call->loc, loc1);
                        num2loc(q->ok_call->loc, loc2);
                        fprintf(err, "Warning %6s %s: rewrite %s %s->%s %s generated DUPE. Maybe %s was active in contest?\n", 
                            c->call, band_str, q->r_call->call, loc1, q->ok_call->call, loc2, q->r_call->call);
                    }

                } else if (q->wwl_outside) {
                    fprintf(htm, "<a href='#WNC'>WNC</a>");
                    err_wnc = 1;
                } else if (q->invalid_rsts) {
                    fprintf(htm, "<a href='#IRSTS'>IRSTS</a>");
                    err_irsts = 1;
                } 
                fprintf(htm, "</td>\n");

                if (q->ok_call->first_qso)
                    fprintf(htm, "<td>Y</td>\n ");
                else
                    fprintf(htm, "<td></td>\n");

                if (q->foundby)
                    fprintf(htm, "<td>%s</td>\n", q->foundby);
                else
                    fprintf(htm, "<td></td>\n");

                fprintf(htm, "<td align='right'>%d</td>\n", -dis);
                if (phase) fprintf(htm, "<td>#%d</td>", q->phase);
                fprintf(htm, "</tr>\n");
                lost += dis;
            }
        }
        fprintf(htm, "</table>\n");

        if (err_rnz || err_snz || err_nil || err_dupe10 || err_wnc || err_irsts || err_ddupe) {
            fprintf(htm, "Description of other errors:<br>\n");
            if (err_rnz)    fprintf(htm, "<a name='RNZ'>RNZ</a> = Received number is zero<br>\n");
            if (err_snz)    fprintf(htm, "<a name='SNZ'>SNZ</a> = Sent number is zero<br>\n");
            if (err_nil)    fprintf(htm, "<a name='NIL'>NIL</a> = Not in the log<br>\n");
            if (err_dupe10) fprintf(htm, "<a name='DUPEx10'>DUPEx10</a> = Computed duplicate contact with good callsign. 10 x penalty<br>\n");
            if (err_wnc)    fprintf(htm, "<a name='WNC'>WNC</a> = Received big WWL square is outside of the country<br>\n");
            if (err_irsts)  fprintf(htm, "<a name='IRSTS'>IRSTS</a> = Invalid SENT RST<br>\n");
            if (err_ddupe)  fprintf(htm, "<a name='DDUPE'>DDUPE</a> = Declared DUPE or zero-point QSO<br>\n");
            fprintf(htm, "<br>\n");
        }    
        
        fprintf(htm, "Claimed score: %6d<br>\n", c->score_claimed);
        if (c->score_claimed != score + lost)
            fprintf(htm, "Correct claimed score: %d<br>\n", score + lost);
        fprintf(htm, "Sum of lost points: %5d<br>\n", -lost);
        fprintf(htm, "Corrected score: %6d<br>\n", score);
        fprintf(htm, "Errors: %2.1f%%<br>\n", (lost + score) ? ((float) (lost * 100)) / (lost + score) : 0);
        fprintf(htm, "<br>\nProcessed by <a href='http://tucnak.nagano.cz/wiki/Lx'>%s %s</a><br>\n", PACKAGE_NAME, PACKAGE_VERSION);
        /*
         * if (c->score_claimed!=score+lost) { fprintf(out,
         * "\nIncorrect computed QRB in QSOs:\n\nCall Loc Claimed
         * QRB Correct QRB\n"); q=c->first_qso; while (q) {
         * dis=QRB(c->loc, q->r_call->loc); if (!q->dupe &&
         * dis!=q->claimed_QRB) { num2loc(q->r_call->loc, loc);
         * fprintf(out, "%-14s %6s %4i %4i\n", q->r_call->call,
         * loc, q->claimed_QRB, dis); } q=q->next; } } 
         */
        c->score_lost = lost;
        c->score = score;
        c->sumqrb = sumqrb;
        c->odx = odx;
        c->qsos = qsos;
        if (wwlmult){
            c->mults = g_hash_table_size(multhash);
            c->score *= c->mults;
        }
        /*
         * fprintf(out, "\n For Your Information:\n");
         * fprintf(out, " Call In other logs\n"); q=c->first_qso;
         * while (q) { if (q->ok_call->refs<3) { fprintf(out, "
         * %-20s %d ", q->r_call->call, q->r_call->refs-1);
         * c2=q->ok_call->first_call_same_loc; while (c2) { if
         * (c2!=q->ok_call && c2->refs) fprintf(out, "%s[%d] ",
         * c2->call, c2->refs); c2=c2->next_call_same_loc; }
         * fprintf(out, "\n"); }
         * 
         * q=q->next; } 
         */
        fprintf(htm, "</body>\n</html>\n");
        fclose(htm);
        // printf(".");
	}

	// printf("OK\n");
}


void make_score_list(uint8_t band, char *band_str, uint8_t categ, FILE * res,
	FILE * htm, FILE *fd, FILE *dip, float maxlp, float maxqrp, int islp)
{
	DEBUG_FUNC("make_score_list");

	char s[10], asl[10], pwr[50];
	struct CALL_INFO *c, **c2;
	int num = 0, i = 0;
	char loc[7], *callspace;
	char odx_call[SIZE_CALL + 1];
	int odx_QRB, band_freq;
	float average, errors;
    char lpstr[20];
    int lastscore, lastplace;

	// printf("+ Generating %s files: ", filename);

    strcpy(lpstr, "");
    if (islp == 1) sprintf(lpstr, "_%d", (int)maxlp);
    if (islp == 2) sprintf(lpstr, "_%d", (int)maxqrp);

	for (c = first_call[band]; c; c = c->next){
		if (c->categ != categ) continue;
        int incl = inclcall(c->pwr, maxlp, maxqrp, islp);
//        printf("score: %-6s %3.0f %3.0f %d %d\n", c->call, c->pwr, maxpwr, islp, incl);
        if (!incl) continue;

    	num++;
	}

	if (!num)
		return;

    g_do_check = 1;
	printf("+ Generating result of %s, %s: ", band_str,
		   categs[categ].name);

//    printf("  maxpwr=%f islp=%d strictlp=%d num=%d ", maxpwr, islp, strictlp, num);

	if (band >= BAND_GHZ)
		band_freq = (int) (band2freq[band] * 1000);
	else
		band_freq = (int) band2freq[band];
	fprintf(fd, "\t<category band=\"%u\" operator=\"%s\">\n", band_freq, categs[categ].name);
    fprintf(htm, "<a name=\"%d_%d%s\"></a>\n", band, categ, lpstr);
	fprintf(htm,
			"<h2>Category: %s, %s</h2>\n",
			band_str, categs[categ].name);
	fprintf(htm, "<table class=\"sortable\">\n");
	fprintf(res, "\nCategory: %s, %s\n\n", band_str, categs[categ].name);

    if (dip){
        fprintf(dip,
                "<h2>Category: %s, %s</h2>\n",
                band_str, categs[categ].name);
	    fprintf(dip, "<table class=\"sortable\">\n");
    }


	c2 = (struct CALL_INFO **) malloc(num * sizeof(c));

	for (c = first_call[band]; c; c = c->next){
        if (!inclcall(c->pwr, maxlp, maxqrp, islp)) continue;

		if (c->categ == categ) {
			c2[i] = c;
			i++;
		}
	}

	qsort(c2, num, sizeof(c), compare_fcn);

	fprintf(res,
			"Pl. Call        score  QSO aver. LOC     ASL ODX         QRB   er. Power antenna\n");

	fprintf(htm, "<tr>\n");
	fprintf(htm, "<th>Pl.</th>\n");
	fprintf(htm, "<th>Call</th>\n");
	fprintf(htm, "<th>Score</th>\n");
	fprintf(htm, "<th>QSO</th>\n");
	fprintf(htm, "<th>Aver.</th>\n");
	fprintf(htm, "<th>Locator</th>\n");
	fprintf(htm, "<th>ASL</th>\n");
	fprintf(htm, "<th>ODX</th>\n");
	fprintf(htm, "<th>QRB</th>\n");
	fprintf(htm, "<th>Err. %%</th>\n");
	fprintf(htm, "<th class=\"sorttable_numeric\">Power</th>\n");
	fprintf(htm, "<th>Antenna</th>\n");
	fprintf(htm, "</tr>\n");

    if (dip){
        fprintf(dip, "<tr>\n");
        fprintf(dip, "<th>Pl.</th>\n");
        fprintf(dip, "<th>Call</th>\n");
        fprintf(dip, "<th>PDF</th>\n");
        fprintf(dip, "</tr>\n");
    }



	// fprintf(htm, "<b>Pl. Call score QSO aver. LOC ASL ODX QRB er. P[W]
	// antenna</b>\n");
    lastscore = -1;
    lastplace = -1;
	for (i = 0; i < num; i++) {
		num2loc(c2[i]->loc, loc);

        if (c2[i]->score != lastscore){
            lastscore = c2[i]->score;
            lastplace = i + 1;
        }
        if (c2[i]->score == 0)
            strcpy(s, "");
        else
    		sprintf(s, "%i.", lastplace);

		if (strlen(s) < 4)
			strcat(s, space + strlen(space) + strlen(s) - 4);
		if (strlen(c2[i]->call) < 10)
			callspace = space + strlen(space) + strlen(c2[i]->call) - 10;
		else
			callspace = strchr(space, 0);

		if (c2[i]->asl)
			sprintf(asl, "%4i", c2[i]->asl);
		else
			strcpy(asl, "   ?");

        if (fpower){
            if (inclcall(c2[i]->pwr, maxlp, maxqrp, 2)) 
                strcpy(pwr, "    A");
            else if (inclcall(c2[i]->pwr, maxlp, maxqrp, 1)) 
                strcpy(pwr, "    B");
            else
                strcpy(pwr, "    C");
        }else{
            if (c2[i]->pwr)
                float_print(c2[i]->pwr, pwr);
            else
                strcpy(pwr, "   ?W");
        }

		if (!*c2[i]->ant)
			strcpy(c2[i]->ant, "?");

		*odx_call = '\0';
		odx_QRB = 0;

		if (!c2[i]->odx) {
			strcpy(odx_call, "NONE");
			odx_QRB = 0;
		} else {
			strcpy(odx_call, c2[i]->odx->r_call->call);
			odx_QRB = c2[i]->odx->qrb;
		}
		
		average = (float) (c2[i]->qsos ? ((float) c2[i]->sumqrb) / c2[i]->qsos : 0);
		errors = (c2[i]->score_lost + c2[i]->score) ?
			(((float) (c2[i]->score_lost) * 100) / (c2[i]->score_lost + c2[i]->score)) : 0;

		fprintf(res,
				"%s%-10s %6i %4i %5.1f %s %s %-10s %4i %5.1f %s %-20s\n",
				s, c2[i]->call, c2[i]->score, c2[i]->qsos,
				average, loc, asl, odx_call, odx_QRB,
				errors, pwr, c2[i]->ant);

		fprintf(htm,
				"<td align=right>%i</td>\n",
				c2[i]->qsos);
		fprintf(htm,
				"<td align=right>%5.1f</td>\n",
				average);
		fprintf(htm,
				"<td>%s</td>\n", loc);
		fprintf(htm,
				"<td align=right>%s</td>\n",
				asl);
		fprintf(htm,
				"<td>%s</td>\n",
				odx_call);
		fprintf(htm,
				"<td align=right>%i</td>\n",
				odx_QRB);
		fprintf(htm,
				"<td align=right>%5.1f</td>\n",
				errors);
		fprintf(htm,
				"<td align=right>%s</td>\n",
				pwr);
		fprintf(htm,
				"<td>%s</td>\n",
				c2[i]->ant);
		fprintf(htm, "</tr>\n");

		/*
		 * fprintf(htm, "%s<a href=\"%s\">%s</a>%s %6i %4i %5.1f %s %s
		 * %-10s %4i %5.1f %s %-20s\n", s,
		 * c2[i]->filename_err+strlen(cfg_epath)+1, c2[i]->call,
		 * callspace, c2[i]->score, c2[i]->qsos,
		 * (float)(c2[i]->qsos?((float)c2[i]->score)/c2[i]->qsos:0), loc,
		 * asl, odx_call, odx_QRB,
		 * (c2[i]->score_lost+c2[i]->score)?((float)(c2[i]->score_lost)*100)/(c2[i]->score_lost+c2[i]->score):0, 
		 * pwr, c2[i]->ant); 
		 */
		 
		 fprintf(fd, "\t\t<station place=\"%u\" call=\"%s\" score=\"%u\" qsos=\"%u\" average=\"%.2f\" wwl=\"%s\" asl=\"%u\" odx=\"%s, %u\" errors=\"%.2f\" power=\"%.6f\" antenna=\"%s\" />\n",
		 	i + 1, c2[i]->call, c2[i]->score, c2[i]->qsos, average, loc, c2[i]->asl, odx_call, odx_QRB,
		 	errors, c2[i]->pwr, c2[i]->ant);

		 if (dip && i+1 <= diplomplace){
			 char *file = make_diplom(categ, c2[i]->call, i+1, band_str);
             if (file){
                 fprintf(dip, "<tr>\n");
                 fprintf(dip,
                        "<td align=right>%s</td>\n",
                        s);
                 fprintf(dip,
                        "<td>%s</td>\n",
                        c2[i]->call);
                 fprintf(dip,
                        "<td><a href=\"%s\">%s</a></td>\n",
                        file, file);
                 fprintf(dip, "</tr>\n");
             }
         }
	}

	// fclose(out);

	free(c2);
	
	fprintf(fd, "\t</category>\n");

	fprintf(htm, "</table>\n<hr><br>");
    if (dip) fprintf(dip, "</table>\n<hr><br>");
	fprintf(res, "\n");

	printf("%i stations\n", num);
}

int compare_str(const void *a, const void *b){
    char **ca = (char **)a;
    char **cb = (char **)b;
    return strcmp(*ca, *cb);
}

void make_check_list(uint8_t band, FILE *res, FILE *htm){
	DEBUG_FUNC("make_check_list");
    int l, num, i;
    char text[256];
    char **sorted;

	struct CALL_INFO *c;
            
    if (band >= BAND_GHZ) {
        if (band >= BAND_GHZ_ZERO)
            sprintf(text, "%2.0f GHz", band2freq[band]);
        else
            sprintf(text, "%2.1f GHz", band2freq[band]);
    } else
        sprintf(text, "%2.0f MHz", band2freq[band]);

    fprintf(htm, "<h2>Checklogs used for %s:</h2><p>\n", text);
	fprintf(res, "\nChecklogs used for %s:\n\n", text);

    num = 0;
	for (c = first_call[band]; c; c = c->next){
		if (c->categ != 0) continue;
        if (c->first_qso == NULL) continue;
        num++;
	}

    sorted = malloc(num * sizeof(char *));
	
    i = 0;
    for (c = first_call[band]; c; c = c->next){
		if (c->categ != 0) continue;
        if (c->first_qso == NULL) continue;
        sorted[i++]=c->call;
	}

    
    qsort(sorted, num, sizeof (char *), compare_str);

	
    l=0;
	for (i=0; i<num; i++){
        fprintf(htm, "%s ", sorted[i]);
        l+=fprintf(res, "%s ", sorted[i]);
        if (l>=70){
            l=0;
            fprintf(res, "\n");
        }
	}
    free(sorted);
	
    fprintf(htm, "</p><hr><br>\n");
	fprintf(res, "\n\n");
    

}

int inclcall(float pwr, float maxlp, float maxqrp, int islp){
    if (strictlp){
        switch (islp){
            case 0: // qro
                if (pwr == 0.0) return 1;
                if (pwr <= maxlp) return 0;
                break;
            case 1: // lp
                if (pwr == 0.0) return 0;
                if (pwr > maxlp) return 0;   
                if (pwr <= maxqrp) return 0;   
                break;
            case 2: // qrp
                if (pwr == 0.0) return 0;
                if (pwr > maxqrp) return 0;   
                break;
        }

    }else{
        switch (islp){
            case 0: // qro
                return 1;
                break;
            case 1: // lp
                if (pwr > maxlp) return 0;
                if (pwr == 0.0) return 0;
                break;
            case 2: // qrp
                if (pwr > maxqrp) return 0;
                if (pwr == 0.0) return 0;
                break;
        }
    }
    return 1;
}
